require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
class Game
  attr_accessor :name
  attr_reader :players
  
  def initialize (mylistname)
    @name = mylistname.capitalize
    @players = []
  end
    
  def load_players(from_file)
    File.readlines(from_file).each do |line|
      add_player(Player.from_csv(line))
    end
  end
  
  def save_high_scores(to_file="high_scores.txt")
    File.open(to_file, "w") do |file|
      file.puts "#{@name} High Scores:"

      @players.sort.each do |p|
        file.puts high_score_entry(p)
      end
    end
  end
  
  def add_player(player)
    @players << player
  end
  
  def play(rounds)
    puts "#{@name}'s game:"
    puts @players.sort
    
    treas = TreasureTrove::TREASURES
    puts "\nThere are #{treas.size} treasures to be found."
    
    treas.each do |t|
      puts "#{t.name} is worth #{t.points} points"
    end
    
    1.upto(rounds) do |round|
      puts "\nRound number #{round}:"
      @players.each do |player|
        GameTurn.take_turn(player)
        puts player
      end
    end
  end
  
  def print_name_and_health(pla)
    puts "#{pla.name} has a health of #{pla.health}"
  end
  
  def total_points
    @players.reduce(0) do |sum, player|
      sum + player.points
    end
  end
  
  def high_score_entry(player)
    formatted_name = player.name.ljust(20, '.')
    "#{formatted_name} #{player.score}"
  end
  
  def print_stats
    puts "\n#{@name}'s Stats:"
    
    puts "#{total_points} total points earned"
    @players.sort.each do |p|
      puts "\n#{p.name}'s point totals:"
      p.each_found_treasure do |t|
        puts "#{t.points} total #{t.name} points"
      end
      puts "#{p.points} grand total points"
    end
    
    strong, weak = @players.partition { |player| player.strong? }
    
    puts "\nStrong:"
    strong.each {|p| self.print_name_and_health(p)}
#    puts strong.sort
    
    puts "\nWeak:"
    weak.each {|p| self.print_name_and_health(p)}
#    puts weak.sort
    
    puts "\nHigh Scores"
    @players.sort.each do |p|
      puts high_score_entry(p)
    end
  end
  
  def to_s
    "There are #{@players.size} players in #{name}."
  end
  
end
end